@@ -1,6 +1,6 @@
 package OpenInteract::Handler::Page;
 
-# $Id: Page.pm,v 1.13 2001/11/27 12:08:36 lachoy Exp $
+# $Id: Page.pm,v 1.30 2002/01/18 20:07:49 lachoy Exp $
 
 use strict;
 use Class::Date ();
@@ -9,7 +9,7 @@ use OpenInteract::Handler::GenericDispatcher qw( DEFAULT_SECURITY_KEY );
 use SPOPS::Secure qw( :level );
 
 @OpenInteract::Handler::Page::ISA     = qw( OpenInteract::CommonHandler  SPOPS::Secure );
-$OpenInteract::Handler::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.13 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract::Handler::Page::VERSION = sprintf("%d.%02d", q$Revision: 1.30 $ =~ /(\d+)\.(\d+)/);
 $OpenInteract::Handler::Page::author            = 'chris@cwinters.com';
 $OpenInteract::Handler::Page::default_method    = 'show';
 @OpenInteract::Handler::Page::forbidden_methods = ();
@@ -23,9 +23,10 @@ $OpenInteract::Handler::Page::default_method    = 'show';
 use constant DEFAULT_EXPIRE  => '365D';
 
 # Use this to check whether the file retrieved is displayable in the
-# browser; others (pdf, ps, mov, etc.) get sent to the user directly
+# browser and in the normal template setup; others (pdf, ps, mov,
+# etc.) get sent to the user directly 
 
-my %DISPLAY_TYPES = map { $_ => 1 } ( 'text/html', 'text/plain', 'text/xml' );
+my %DISPLAY_TYPES = map { $_ => 1 } ( 'text/html' );
 
 # Use this to separate your single document into multiple pages
 
@@ -36,36 +37,80 @@ sub MY_PACKAGE                 { return 'base_page' }
 sub MY_HANDLER_PATH            { return '/Page' }
 sub MY_OBJECT_TYPE             { return 'page' }
 sub MY_OBJECT_CLASS            { return OpenInteract::Request->instance->page }
-sub MY_SEARCH_FIELDS           { return () }
+
+sub MY_SEARCH_FORM_TEMPLATE    { return 'page_search_form' }
+sub MY_SEARCH_FORM_TITLE       { return 'Search Pages' }
+sub MY_ALLOW_SEARCH_FORM       { return 1 }
+
+sub MY_SEARCH_RESULTS_TEMPLATE { return 'page_search_results' }
+sub MY_SEARCH_RESULTS_TITLE    { return 'Page Search Results' }
+sub MY_SEARCH_FIELDS           { return qw( title author keywords main_template ) }
+sub MY_SEARCH_FIELDS_EXACT     { return qw( is_active ) }
+sub MY_ALLOW_SEARCH            { return 1 }
+
+sub MY_ALLOW_CREATE            { return 1 }
+sub MY_OBJECT_CREATE_SECURITY  { return SEC_LEVEL_WRITE }
+
 sub MY_EDIT_RETURN_URL         { return '/' }
+sub MY_EDIT_FAIL_TASK          { return 'show' }
 sub MY_EDIT_DISPLAY_TASK       { return 'show' }
 sub MY_EDIT_FIELDS             { return qw( location title author keywords
                                             boxes main_template notes content
                                             storage content_location ) }
 sub MY_EDIT_FIELDS_TOGGLED     { return qw( is_active template_parse ) }
 sub MY_EDIT_FIELDS_DATE        { return qw( active_on expires_on ) }
-sub MY_ALLOW_SEARCH_FORM       { return 1 }
-sub MY_ALLOW_SEARCH            { return 1 }
-sub MY_ALLOW_CREATE            { return 1 }
-sub MY_OBJECT_CREATE_SECURITY  { return SEC_LEVEL_WRITE }
-sub MY_ALLOW_SHOW              { return 1 }
 sub MY_ALLOW_EDIT              { return 1 }
+
+sub MY_REMOVE_FAIL_TASK        { return 'show' }
+sub MY_REMOVE_DISPLAY_TASK     { return 'search_form' }
 sub MY_ALLOW_REMOVE            { return 1 }
+
 sub MY_ALLOW_NOTIFY            { return 1 }
 sub MY_ALLOW_WIZARD            { return undef }
 
 
+sub admin   { return $_[0]->actions( $_[1] ) }
+sub actions {
+    my ( $class, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $R->{page}{title} = "Page Administrative Actions";
+    return $R->template->handler( {}, {},
+                                  { name => 'base_page::admin_actions' } );
+}
+
 # Overrides entry in OpenInteract::Handler::GenericDispatcher
 
 sub _get_task {
     my ( $class ) = @_;
     my $R = OpenInteract::Request->instance;
-    return 'show'  if ( $R->{path}{full}->[0] ne 'Page' );
-    return lc shift @{ $R->{path}{current} } ||
-           $OpenInteract::Handler::Page::default_method;
+    my ( $task );
+    $task = 'show'  if ( $R->{path}{full}->[0] !~ /^page$/i );
+    $task ||= lc shift @{ $R->{path}{current} } ||
+              $OpenInteract::Handler::Page::default_method;
+    $R->DEBUG && $R->scrib( 1, "Task found from _get_task: [$task]" );
+    return $task;
 }
 
 
+sub _search_criteria_customize {
+    my ( $class, $criteria ) = @_;
+    my $R = OpenInteract::Request->instance;
+    my $table = $R->page->table_name;
+
+    # Administrators can pick whether to find active pages or not,
+    # everyone else has to only search active pages. (Note: don't use
+    # '_read_field_toggled' here because we want to ignore the
+    # parameter if it wasn't chosen.
+
+    if ( $R->{auth}{is_admin} ) {
+        my $active_choice = $class->_read_field( $R->apache, 'is_active' );
+        $criteria->{"$table.is_active"} = $active_choice if ( $active_choice );
+    }
+    else {
+        $criteria->{"$table.is_active"} = 'yes';
+    }
+}
+
 
 # Retrieve all directories, expanding the one we were asked to (if at
 # all). Note that these are just the objects in the database, although
@@ -76,7 +121,7 @@ sub directory_list {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
 
-    my $selected_dir = $R->apache->param( 'selected_dir' );
+    my $selected_dir = $class->_read_field( $R->apache, 'selected_dir' );
     my $params = { selected_dir => $selected_dir,
                    error_msg    => $p->{error_msg},
                    status_msg   => $p->{status_msg} };
@@ -113,13 +158,47 @@ sub _edit_customize {
 
     my %opts = ();
 
-    # If the user changed the location, then we need to set the ID so
-    # the UPDATE works properly.
+    # See if the upload should be there -- note that
+    # _handle_uploaded_file() sets the MIME type for us and sets the
+    # filename, size and type reported by the upload in tmp_upload
 
-    if ( $page->{location} ne $old_data->{location} ) {
+    my $has_upload = $class->_read_field_toggled( $R->apache, 'use_upload' );
+    if ( $has_upload eq 'yes' ) {
+        $R->DEBUG && $R->scrib( 1, "Handling file upload" );
+        my ( $type, $msg ) = $class->_handle_uploaded_file( $page );
+        return ( $type, $msg ) if ( $type );
+    }
+
+    # See if we're using the 'simple' form, and if so set the location
+    # of the file from the filename uploaded (after cleaning)
+
+    my $form_type = $class->_read_field( $R->apache, 'form_type' ) || 'normal';
+    if ( $form_type eq 'simple' ) {
+        my $dir = $class->_read_field( $R->apache, 'directory' );
+        unless ( $dir ) {
+            my $error_message = "Failed to upload file. No directory specified.";
+            return ( ERROR, $error_message );
+        }
+        $dir =~ s|/$||;
+        my $filename = $R->page->clean_location( $page->{tmp_upload}{filename} );
+        $page->{location} = join( '', $dir, $filename );
+        $R->DEBUG && $R->scrib( 1, "Set location from uploaded file to [$page->{location}]" );
+    }
+
+    # If we're not using the simple form, check to see if the user
+    # changed the location, then we need to set the ID so the UPDATE
+    # works properly.
+
+    elsif ( $page->{location} ne $old_data->{location} ) {
         $opts{use_id} = $old_data->{location};
     }
 
+    # Ensure that the location is clean
+
+    $R->DEBUG && $R->scrib( 1, "Location before clean [$page->{location}]" );
+    $page->{location} = $R->page->clean_location( $page->{location} );
+    $R->DEBUG && $R->scrib( 1, "Location after clean [$page->{location}]" );
+
     # Default the expires_on field
 
     unless ( $page->{expires_on} ) {
@@ -127,41 +206,15 @@ sub _edit_customize {
         $page->{expires_on} = $expire_date->strftime( '%Y-%m-%d' );;
     }
 
-    # See if the upload should be there
-
-    my $has_upload = $class->_read_field_toggled( $R->apache, 'use_upload' );
-    if ( $has_upload eq 'yes' ) {
-        $R->DEBUG && $R->scrib( 1, "User is requesting content from uploaded file..." );
-        my $upload  = $R->apache->upload( 'content_upload' );
-        unless ( $upload ) {
-            my $error_msg = 'You checked off that you wanted to upload a ' .
-                            'file but did not upload one. Why are you teasing me?';
-            my %error_opts = ( method => 'show', error_msg => $error_msg, edit => 1 );
-            return ( ERROR, \%error_opts );
-        }
-        $R->DEBUG && $R->scrib( 1, "Upload seems to be retrieved ok. Here is some info:\n",
-                                   "Filename: (", $upload->filename, ") Size: (",
-                                   $upload->size, ") Type: (", $upload->type, ")" );
-        $page->{size}      = $upload->size;
-        $page->{mime_type} = $upload->type;
-        if ( $class->_is_displayable( $page->{mime_type} ) ) {
-            my $fh = $upload->fh;
-            local $/ = undef;
-            my $content = <$fh>;
-            $page->{content} = \$content;
-        }
-        else {
-            $page->{content} = $upload->fh;
-        }
-    }
-
     $page->{mime_type} ||= 'text/html';
 
     # Non-displayable docs always get saved to the filesystem (for
-    # now); also, remove 'content' from the list of fields to be
-    # processed by the FullText indexer
+    # now); for these docs we also need to remove 'content' from the
+    # list of fields to be processed by the FullText indexer
 
     unless ( $class->_is_displayable( $page->{mime_type} ) ) {
+        $R->DEBUG && $R->scrib( 1, "Not displayable [$page->{mime_type}];",
+                                   "don't scan content for index" );
         $page->{is_file} = 'yes';
         if ( $page->CONFIG->{fulltext_field} ) {
             $opts{fulltext_field} = [ grep ! /^content$/, @{ $page->{fulltext_field} } ];
@@ -172,65 +225,51 @@ sub _edit_customize {
 }
 
 
-
 # Why do we set the content-type when returning errors? See note on
 # error content-type forcing in POD...
 
 sub show {
     my ( $class, $p ) = @_;
     my $R = OpenInteract::Request->instance;
-    my $params = { error_msg   => $p->{error_msg},
-                   status_msg  => $p->{status_msg} };
+    $R->DEBUG && $R->scrib( 1, "Edit status of page: [edit: $p->{edit}] ",
+                               "[new object: $p->{is_new_object}]" );
 
-    # Get the actual location requested -- this can be passed in
-    # directly, specified in a parameter or taken from the URL.
+    my $page = $p->{page};
+    $page  ||= $R->page->new()  if ( $p->{is_new_object} );
 
-    my $location = $class->_clean_location( $class->_find_location( $p ) );
+    my ( $location );
 
     # Try and find a page object (just the metadata) matching with our
-    # location. Note that _find_page_object() will also treat
+    # location. Note that page_by_location() will also treat
     # $location as a directory request, where appropriate
 
-    my $page = $p->{page} || eval { $class->_find_page_object( $location ) };
-    if ( $@ ) {
-        $R->{page}{content_type} = 'text/html';
-        $R->DEBUG && $R->scrib( 1, "Could not retrieve page. Error ($@)" );
-        if ( $@ =~ /^security/ ) {
-            my $admin_email = $R->CONFIG->{mail}{admin_email};
-            $R->throw({ code => 303 });
+    unless ( $page ) {
+        $location = $class->_get_location( $p );
+        my $item = eval { $R->page->page_by_location( $location ) };
+        if ( $@ ) {
+            $class->_fail_page_fetch( $location, $@ );
         }
-        elsif ( $@ =~ /^access/ ) {
-            return "<h2>Cannot Access</h2><p>Failure accessing page.</p>";
+        if ( $item and $item->isa( $R->page_directory ) ) {
+            return $class->_run_directory_handler( $item, $location, $p );
         }
+        $page = $item;
     }
 
-    my $do_edit = $p->{edit} || $R->apache->param( 'edit' );
+    # Bail if we're not creating a new page and haven't found a page
+    # to display yet
+
+    my $do_edit = $p->{edit} || $class->_read_field( $R->apache, 'edit' );
     unless ( $page or $do_edit ) {
         $R->{page}{content_type} = 'text/html';
+        $location ||= $page->{location};
         $R->throw({ code => 314, system_msg => $location });
     }
 
-    # Now we have a page; just check to see if we were instructed to
-    # display the editable form for this page, and if so ensure this
-    # user can do so.
+    # See if we're supposed to edit
 
     if ( $do_edit ) {
-        $page ||= $R->page->new;
-
-        # If this is an editable doc, fetch the content, otherwise,
-        # mark it as non-editable to the form
-
-        $page->{storage} ||= 'file';
-        if ( $class->_is_displayable( $page->{mime_type} ) ) {
-            $page->fetch_content;
-        }
-        else {
-            $params->{non_editable} = 1;
-        }
-        $R->DEBUG && $R->scrib( 1, "This page should be in an editable form" );
-        $params->{page} = $page;
-        $R->{page}{title}   = 'Edit a Document';
-        return $R->template->handler( {}, $params, { name => 'base_page::page_form' } );
+        $page->{location} ||= $class->_get_location;
+        return $class->show_editable_page( $page, $p );
     }
 
     # If we specified that we're going to send a separate file to the
@@ -238,55 +277,131 @@ sub show {
     # quit processing
 
     unless ( $class->_is_displayable( $page->{mime_type} ) ) {
-        $R->{page}{content_type}   = $page->{mime_type};
-        $R->{page}{send_file}      = join( '', $R->CONFIG->get_dir( 'html' ), $page->{location} );
-        $R->{page}{send_file_size} = $page->{size};
-        $R->DEBUG && $R->scrib( 1, "File being retrieved is not directly displayable.",
-                                   "Set 'send_file' to ($page->{location})" );
-        return undef;
+        return $class->show_nondisplayable_page( $page, $p );
     }
 
-    # We have a page and we can display it, so deal with the fact that
-    # it may be an alias and then grab the relevant content
+    return $class->show_displayable_page( $page, $p );
+}
 
-    while ( $page->{storage} eq 'alias' ) {
-        $page = eval { $class->_find_page_object( $page->{content_location} ) };
-        if ( $@ or ! $page ) {
-            $R->scrib( 0, "Location for alias ($page->{content_location})",
-                          "wasn't found. ($@)" );
-            return "Request was for an alias, but aliased page unavailable.";
-        }
-    }
+sub show_displayable_page {
+    my ( $class, $page, $p ) = @_;
+    die "Must pass page to display\n" unless ( ref $page );
 
-    $page->fetch_content;
+    my $R = OpenInteract::Request->instance;
 
-    # Otherwise, ensure the page is active, set metadata and send it
-    # off to be displayed
+    # Ensure the page is active
 
-    $R->DEBUG && $R->scrib( 1, "Display ($page->{location}) as normal HTML" );
     unless ( $class->_is_active( $page ) ) {
         $R->DEBUG && $R->scrib( 1, "Page is not currently active; return error" );
         $R->{page}{title} = 'Page not active';
         return '<h2 align="center">Not Active</h2><p>Sorry, this page is not active.</p>';
     }
 
+    # Follow the alias chain to its end
+
+    while ( $page->{storage} eq 'alias' ) {
+        $page = eval { $R->page->page_by_location( $page->{content_location} ) };
+        if ( $@ or ! $page ) {
+            $R->scrib( 0, "Location for alias [$page->{content_location}]",
+                          "wasn't found: $@" );
+            return "Request was for an alias, but aliased page unavailable.";
+        }
+    }
+
+    $R->DEBUG && $R->scrib( 1, "Display [$page->{location}] as normal HTML" );
+    $page->fetch_content;
+
+    # Use page metadata to modify display
+
     $R->{page}{title} = $page->{title};
-
-    # Allows the page to define the main template it will use; if
-    # the page doesn't define one then the main UI module will use
-    # the default for the current theme
-
     $R->{page}{_template_name_} = $page->{main_template};
-
     $class->_add_object_boxes( $page, $p );
 
-    # You can split your page into multiple viewable pages -- see
-    # _split_pages() for more info
+    # Grab content we're actually going to show
 
     my $display_content = $class->_split_pages( $page );
 
     return $display_content if ( $page->{template_parse} eq 'no' );
-    return $R->template->handler( {}, $params, { text => \$display_content } );
+    return $R->template->handler( {}, $p, { text => \$display_content } );
+}
+
+
+sub show_editable_page {
+    my ( $class, $page, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $page ||= $R->page->new;
+
+
+    # If this is an editable doc, fetch the content, otherwise,
+    # mark it as non-editable to the form
+
+    $page->{storage} ||= 'file';
+    if ( $class->_is_displayable( $page->{mime_type} ) ) {
+        $page->fetch_content if $page->is_saved;
+    }
+    else {
+        $p->{non_editable} = 1;
+    }
+    $R->DEBUG && $R->scrib( 1, "This page should be in an editable form;",
+                               "uneditable content status [$p->{non_editable}]" );
+    $p->{page} = $page;
+
+    my $tmpl_name = 'page_form';
+
+    # Check and see if this is a request to upload the page using the
+    # 'simple' form
+
+    my $form_type = $p->{form_type} ||
+                    $class->_read_field( $R->apache, 'form_type' ) ||
+                    'normal';
+    if ( $form_type eq 'simple' ) {
+        $tmpl_name = 'page_form_simple';
+        $p->{directory} = $page->{location};
+    }
+
+    $R->{page}{title}   = 'Edit a Document';
+    return $R->template->handler( {}, $p, { name => "base_page::$tmpl_name" } );
+}
+
+
+sub show_nondisplayable_page {
+    my ( $class, $page ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $R->{page}{content_type}   = $page->{mime_type};
+    $R->{page}{send_file}      = join( '', $R->CONFIG->get_dir( 'html' ), $page->{location} );
+    $R->{page}{send_file_size} = $page->{size};
+    $R->DEBUG && $R->scrib( 1, "File being retrieved is not directly displayable.",
+                               "Set 'send_file' to [$page->{location}]" );
+    return undef;
+}
+
+
+sub _fail_page_fetch {
+    my ( $class, $location, $error ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $R->{page}{content_type} = 'text/html';
+    $R->DEBUG && $R->scrib( 1, "Could not retrieve page. Error [$error]" );
+    if ( $error =~ /^security/ ) {
+        $R->throw({ code => 303 });
+    }
+    elsif ( $error =~ /^access/ ) {
+        return "<h2>Cannot Access</h2><p>Failure accessing page.</p>";
+    }
+    return "<h2>Unknown Error</h2><p>Could not fetch page -- unknown failure.</p>";
+}
+
+
+sub _run_directory_handler {
+    my ( $class, $page_directory, $location, $p ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $p->{page_directory} = $page_directory;
+    $p->{directory}      = $location;
+    my ( $dir_class, $dir_method ) = $R->lookup_action( $page_directory->{action} );
+    if ( $dir_class and $dir_method ) {
+        return $dir_class->$dir_method( $p );
+    }
+    die "Tried to execute directory handler for ($page_directory->{directory}) ",
+        "but there is no mapping for ($page_directory->{action}).\n";
 }
 
 
@@ -301,103 +416,27 @@ sub _is_displayable {
 }
 
 
-# Find the location from whatever is available -- passed parameters,
-# GET/POST parameters, or the original path
+# Grab the location from whatever is available -- passed parameters,
+# GET/POST parameters, or the original path. Once found, clean it up
 
-sub _find_location {
+sub _get_location {
     my ( $class, $p ) = @_;
-    return $p->{page}{location} if ( $p->{page} );
     my $R = OpenInteract::Request->instance;
-    return $p->{location} ||
-           $R->apache->param( 'location' ) ||
-           $R->{path}{original};
-}
+    my ( $location );
+    $location = $p->{page}{location} if ( $p->{page} );
+    unless ( $location ) {
+        $location   = $p->{location} ||
+                      $class->_read_field( $R->apache, 'location' );
+        $location ||= $R->{path}{original} if ( $R->{path}{original} !~ m|^/page\b|i );
+    }
 
+    # Clean up the location
 
-# Security -- remove all '.' from the beginning of the location
-# requested so people don't try to go up the directory tree. Also
-# remove any two-dot sequence.
-#
-# In the future we might flag these as bad requests (die from here)
-# and simply bail with a stern scoling.
-
-sub _clean_location {
-    my ( $class, $location ) = @_;
-    return undef unless ( $location );
-    $location =~ s/^\.+//;
-    $location =~ s/\.\./_/;
+    $location = $R->page->clean_location( $location );
     return $location;
 }
 
 
-# Find object with $location in the database. We also try to do the
-# work so that you can request a directory index ('home',
-# 'index.html', etc.);
-
-sub _find_page_object {
-    my ( $class, $location ) = @_;
-    my $R = OpenInteract::Request->instance;
-    $R->DEBUG && $R->scrib( 1, "Trying to retrieve object with location ($location)" );
-
-    # Chop off any query strings and put in a case-consistent format
-
-    $location    = $class->_remove_query_string( $location );
-    $location    = lc $location;
-
-    # Just like 'DirectoryIndex' in Apache...
-
-    my $index_names = $R->CONFIG->{action}{page}{directory_index} || [];
-    $R->DEBUG && $R->scrib( 1, "Using the following for index names: ",
-                               join( ', ', @{ $index_names } ) );
-
-    my @locations = ( $location );
-    if ( $location =~ m|/$| ) {
-        $R->DEBUG && $R->scrib( 1, "Explicit directory request; add indexes" );
-        push @locations, map { "$location$_" } @{ $index_names };
-    }
-    elsif ( $location !~ /\.\w+$/ ) {
-        $R->DEBUG && $R->scrib( 1, "Location has no extension; add indexes " ),
-        push @locations, map { "$location/$_" } @{ $index_names };
-    }
-    else {
-        my ( $sans_extension );
-        ( $sans_extension = $location ) =~ s/\.\w+$//;
-        $R->DEBUG && $R->scrib( 1, "Also check location without the extension ",
-                                  "using $sans_extension" );
-        push @locations, $sans_extension;
-    }
-
-    my ( $page );
-    my ( $error_type );
-    foreach my $location ( @locations ) {
-        $page = eval { $R->page->fetch( $location ) };
-        if ( $@ ) {
-            $R->scrib( 0, "Encountered error trying to retrieve ($location);",
-                          "continuing with other locations." );
-            $error_type = SPOPS::Error->get->{type};
-        }
-        if ( $page ) {
-            $R->DEBUG && $R->scrib( 1, "Found matching location: ($page->{location})" );
-            return $page;
-        }
-    }
-
-    # Returning undef means there were no errors, we just didn't find the page
-
-    return undef unless ( $error_type );
-    die $error_type;
-}
-
-
-# Lop off the query string from $text
-
-sub _remove_query_string {
-    my ( $class, $text ) = @_;
-    $text =~  s|^(.*)\?.*$|$1|;
-    return $text;
-}
-
-
 # A page can have one or more tags that declare it wants itself split
 # into multiple pieces for display. This routine does the
 # splitting. This is still under development...
@@ -411,7 +450,7 @@ sub _split_pages {
 
     if ( $page->{content} =~ /$PAGE_SEPARATOR/ ) {
         my @text_pages      = split /$PAGE_SEPARATOR/, $page->{content};
-        my $page_num        = $R->apache->param( 'pagenum' ) || 1;
+        my $page_num        = $class->_read_field( $R->apache, 'pagenum' ) || 1;
         my $this_page       =  $text_pages[ $page_num - 1 ];
         my $total_pages     = scalar @text_pages;
         my $current_pagenum = $page_num;
@@ -442,7 +481,7 @@ sub _add_object_boxes {
         my @boxes = split /\s+/, $box_string;
         foreach my $box_name ( @boxes ) {
             next if ( $box_name =~ /^\s*$/ );
-            $R->DEBUG && $R->scrib( 1, "Adding box name ($box_name) from page definition" );
+            $R->DEBUG && $R->scrib( 1, "Adding box name [$box_name] from page definition" );
             my $box_params = { name => $box_name };
             if ( $box_name =~ s/^\-// ) {
                 $box_params->{name}   = $box_name;
@@ -465,22 +504,74 @@ sub _add_object_boxes {
 
 sub _is_active {
     my ( $class, $page ) = @_;
+    my $R = OpenInteract::Request->instance;
 
-    return undef if ( $page->{is_active} eq 'no' );
-    return 1 unless ( $page->{active_on} );
+    if ( $page->{is_active} eq 'no' ) {
+        $R->DEBUG && $R->scrib( 1, "Page NOT active: 'is_active' is false" );
+        return undef;
+    }
+    unless ( $page->{active_on} ) {
+        $R->DEBUG && $R->scrib( 1, "Page is active: there is no 'active_on' date" );
+        return 1;
+    }
 
     my $active  = Class::Date::date([ split /\D/, $page->{active_on} ]);
     my $now     = Class::Date->now;
+
+    if ( $active > $now ) {
+        $R->DEBUG && $R->scrib( 1, "Page NOT active: 'active_on' check failed" );
+        return undef;
+    }
+
+    # OK if there's no expiration date, and the active date is ok
+
+    unless ( $page->{expires_on} ) {
+        $R->DEBUG && $R->scrib( 1, "Page is active: 'active_on' check ok, no expiration" );
+        return 1;
+    }
+
     my $expires = Class::Date::date([ split /\D/, $page->{expires_on} ]);
+    if ( $now > $expires ) {
+        $R->DEBUG && $R->scrib( 1, "Page NOT active: 'expires_on' check failed" );
+        return undef;
+    }
 
-    my $R = OpenInteract::Request->instance;
-    $R->DEBUG && $R->scrib( 1, "Active on: $active; Expires on: $expires" );
-
-    return ( $active <= $now and $now <= $expires );
-
+    return 1;
 }
 
 
+sub _handle_uploaded_file {
+    my ( $class, $page ) = @_;
+    my $R = OpenInteract::Request->instance;
+    $R->DEBUG && $R->scrib( 1, "User is requesting content from uploaded file" );
+    my $upload  = $R->apache->upload( 'content_upload' );
+    unless ( $upload ) {
+        my $error_msg = 'You checked off that you wanted to upload a ' .
+                        'file but did not upload one. Why are you teasing me?';
+        my %error_opts = ( method => 'show', error_msg => $error_msg, edit => 1 );
+        return ( ERROR, \%error_opts );
+    }
+    $page->{tmp_upload} = { filename => $upload->filename,
+                            size     => $upload->size,
+                            type     => $upload->type };
+    $R->DEBUG && $R->scrib( 1, "Upload seems to be retrieved ok. Here is some info:\n",
+                               "Filename: [$page->{tmp_upload}{filename}] Size:",
+                               "$page->{tmp_upload}{size}] Type: [$page->{tmp_upload}{type}]" );
+    $page->{size} = $upload->size;
+    if ( $class->_is_displayable( $page->{mime_type} ) ) {
+        my $fh = $upload->fh;
+        local $/ = undef;
+        my $content = <$fh>;
+        $page->{content} = \$content;
+    }
+    else {
+        $page->{content} = $upload->fh;
+    }
+    $page->{mime_type} = $R->page->mime_type_by_extension( $upload->filename ) ||
+                         $page->{tmp_upload}{type};
+    return;
+}
+
 1;
 
 __END__
@@ -535,7 +626,13 @@ L<OpenInteract::CommonHandler|OpenInteract::CommonHandler>
 
 =head1 BUGS
 
-None known.
+B<Date Format Assumptions>
+
+For the active_on/expires_on dates in the page object, we assume the
+date is formatted yyyy*mm*dd. This may not hold true in all cases. It
+would probably be useful to create an SPOPS add-on that creates a
+Class::Date (or Time::Piece, or whatever) object for every date field
+in an object on a fetch().
 
 =head1 TO DO
 
@@ -547,7 +644,7 @@ L<OpenInteract::CommonHandler|OpenInteract::CommonHandler>
 
 =head1 COPYRIGHT
 
-Copyright (c) 2001 intes.net, inc.. All rights reserved.
+Copyright (c) 2001-2002 intes.net, inc.. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
