@@ -1,6 +1,6 @@
 package OpenInteract::SQLInstall::Page;
 
-# $Id: Page.pm,v 1.2 2001/10/29 05:13:01 lachoy Exp $
+# $Id: Page.pm,v 1.4 2001/12/27 20:11:37 lachoy Exp $
 
 # Do installation of SQL for this package
 
@@ -10,8 +10,8 @@ use vars qw( %HANDLERS );
 @OpenInteract::SQLInstall::Page::ISA = qw( OpenInteract::SQLInstall );
 
 my %files = (
-  tables   => [ 'page.sql', 'page_content.sql', 'content_type.sql' ],
-  data     => [ 'content_types.dat', 'page.dat' ],
+  tables   => [ 'page.sql', 'page_content.sql', 'content_type.sql', 'page_directory.sql' ],
+  data     => [ 'content_types.dat', 'page.dat', 'page_directory.dat' ],
   security => [ 'install_security.dat' ],
 );
 
