@@ -8,11 +8,12 @@ CREATE TABLE sys_error (
  filename      varchar(60) not null,
  line          smallint not null,
  action        varchar(25) null,
- user_id       varchar(10) null,
+ user_id       %%USERID_TYPE%% null,
  session_id    varchar(32) null,
  browser       varchar(75) null,
  referer       varchar(150) null,
  error_time    datetime null,
+ url           varchar(150) null,
  notes         text null,
  primary key   ( error_id )
 )
